"""
    Created on 2012-01-20
    @author: jldupont
"""
import zmq
import logging, os


def run(sock_pub=None, sock_sub=None):

    try:
        ctx = zmq.Context()
        sp = ctx.socket(zmq.ROUTER)
        sp.bind(sock_pub)
        logging.info("Ready to receive for publishers on socket: %s" % sock_pub)
    except Exception,e:
        raise Exception("Can't bind to socket for publishers: %s => %s" % (sock_pub, str(e)))
    
    try:
        ctx = zmq.Context()
        ss = ctx.socket(zmq.PUB)
        ss.bind(sock_sub)
        logging.info("Ready for subscribers on socket: %s" % sock_sub)        
    except Exception:
        raise Exception("Can't use socket '%s' for subscribers" % sock_sub)

    
    ppid=os.getppid()
    logging.info("Process pid: %s" % os.getpid())
    logging.info("Parent pid : %s" % ppid)
    logging.info("Starting loop...")
    while True:
        
        _, topic, msg = sp.recv_multipart()
        ss.send_multipart([topic, msg])



"""
    Created on 2012-02-12
    @author: jldupont
"""
import logging, types

def dnorm(d):
    """
    Normalize dictionary
    
    >>> dnorm({"SoMeKeY":"  spaces  "})
    {'somekey': 'spaces'}
    """
    r={}
    for e in d:
        try:    r[e.lower()]=d[e].strip()
        except: r[e.lower()]=d[e]
    return r

def info_dump(d, align):
    fmt="%-"+str(align)+"s : %s"

    if type(d)==types.DictionaryType:        
        for key in d:
            logging.info(fmt % (key, d[key]))
            
    if type(d)==types.ListType:
        for el in d:
            key, value=el
            logging.info(fmt % (key, value))



"""
    Created on 2012-01-20
    @author: jldupont
"""
import zmq
import logging,sys,os, json

def extract(json_mode, specified_topic, iline):
    """
    Extracts topic:msg
    """
    if json_mode:
        try:
            jso=json.loads(iline)
        except:
            return ("warning", "Can't decode json from: %s" % iline)
        
        topic=jso.get("topic", None)
        if topic is None:
            return ("warning","Missing 'topic' key in: %s" % iline)

        topic=str(topic)
        msg=json.dumps(jso)
        return ("ok", (topic, msg))
        
    ### other case...
    msg=iline
    if specified_topic is not None:
        return ("ok", (specified_topic, msg))
    
    ### specified_topic not specified...
    try:
        topic, sep, msg=iline.partition(":")
        assert(sep==":")
        assert(len(topic)>0)
        return ("ok", (topic, msg))
    except:
        return ("warning", "Invalid format, expecting:  'topic: msg'")

    
    

def run(sock_fabric=None, specified_topic=None, json_mode=None, filter_topics=[]):

    ctx = zmq.Context()
    
    try:
        s = ctx.socket(zmq.DEALER)
        s.connect(sock_fabric)
        logging.info("Connected to fabric with socket: %s" % sock_fabric)
    except Exception:
        raise Exception("Can't connect to fabric: %s" % sock_fabric)

    ppid=os.getppid()
    logging.info("Process pid: %s" % os.getpid())
    logging.info("Parent pid : %s" % ppid)
    logging.info("Starting loop...")
    while True:
        
        ### protection against broken pipe
        if os.getppid()!=ppid:
            logging.warning("Parent process terminated... exiting")
            break
        
        iline=sys.stdin.readline().strip()
        
        code, maybe_data=extract(json_mode, specified_topic, iline) 
        if not code.startswith("ok"):
            logging.warning(maybe_data)
            continue
        
        topic, msg=maybe_data

        ## FILTER-OUT
        if topic in filter_topics:
            continue
    
        try:
            s.send_multipart([topic, msg])
        except Exception,e:
            logging.warning("Error sending on topic '%s': %s -- %s" % (topic, msg, str(e)))
            
        try:
            sys.stdout.write("%s: %s\n" % (topic, msg))
        except:
            pass

        
    